# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
import io
from collections import defaultdict
import matplotlib.pyplot as plt
import seaborn as sns
import numpy as np
import pandas as pd
import json

folder = dataiku.Folder("jGYaG5fn")
df = dataiku.Dataset("preferences").get_dataframe()

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
comparisons = {q: {} for q in set(df["question_id"])}
for i in df.index:
    q = df.at[i, "question_id"]
    current, new = df.at[i, "current_approach"], df.at[i, "new_approach"]
    result = df.at[i, "result"]
    if new not in comparisons[q]:
        comparisons[q][new] = {current: result}
    else:
        comparisons[q][new][current] = result

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
ranking = {q: [] for q in set(df["question_id"])}
for i in df.index:
    q = df.at[i, "question_id"]
    current = df.at[i, "current_approach"]
    new = df.at[i, "new_approach"]
    if len(ranking[q]) == 0:
        ranking[q] = [current]
        last_new = None
    if new != last_new:
        j = 0
        while j < len(ranking[q]):
            if comparisons[q][new][ranking[q][j]] == 1:
                j += 1
            else:
                if j == 0:
                    ranking[q] = [new] + ranking[q]
                else:
                    ranking[q] = ranking[q][:j] + [new] + ranking[q][j:]
                break
        else:
            ranking[q].append(new)
    last_new = new

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
for q in ranking:
    approaches = ranking[q]
    break

wins = {m: defaultdict(int) for m in approaches}

for q in ranking:
    for i in range(len(approaches) - 1):
        for j in range(i + 1, len(approaches)):
            wins[ranking[q][j]][ranking[q][i]] += 1

for a1 in wins:
    for a2 in wins[a1]:
        wins[a1][a2] /= len(ranking)

sorted_approaches = sorted(approaches, key=lambda s: sum(wins[s][k] for k in wins[s]))

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
win_matrix = np.zeros((len(approaches), len(approaches)))

for i in range(len(approaches)):
    for j in range(len(approaches)):
        if i != j:
            win_matrix[i, j] = wins[sorted_approaches[i]][sorted_approaches[j]]
    win_matrix[i, i] = None

_ = sns.heatmap(
    win_matrix, annot=True, xticklabels=sorted_approaches, yticklabels=sorted_approaches
).set(title="Win Rate")

with io.BytesIO() as buf:
    plt.savefig(buf)
    folder.upload_data("heatmap.png", buf.getvalue())

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
ranking_df = pd.DataFrame(
    [(q, json.dumps(ranking[q])) for q in ranking], columns=["question_id", "ranking"]
)
dataiku.Dataset("ranking").write_with_schema(ranking_df)
